/* ------------------------------------------------------------------ */
/*      item            : ReplayCommand.hxx
        generated by    : Autobuild (abuild)
        date            : Thu 09 Nov 2023
        category        : header file
        description     : DUECA Communication Object (DCO)
                          automatically generated by dueca-codegen
        codegen version : 111
        language        : C++
        item            : ReplayCommand.dco
        made by         : Rene' van Paassen
        date            : 220110
        description     : Command to control data replay
        copyright       : (c) 2022-2022 TUDelft-AE-C&S
                        : (c) 2022 René van Paassen
        license         : EUPL-1.2
*/

#pragma once
#if !defined(__DCO_NOPACK)
namespace dueca {
class AmorphStore;
class AmorphReStore;
struct DataWriterArraySize;
};
#endif

#if !defined(__DCO_STANDALONE)
namespace dueca {
struct CommObjectDataTable;
};

#include <gencodegen.h>
#if GENCODEGEN != 111
#error "Generated ReplayCommand.hxx too old, please clean with 'make mrproper'"
#endif
#include <dueca/CommObjectTraits.hxx>
#endif
#include <iostream>

#include <inttypes.h>
#include <dueca/DataTimeSpec.hxx>
#include <string>

namespace dueca {
 
/** This command controls the data replay preparation
    */
struct ReplayCommand
{
  /** typedef for internal reference */
  typedef ReplayCommand __ThisDCOType__;

public:
  /** The name of this class. */
  static const char* const classname;


/** Enumerated type for an automatically generated object class */
  enum class Command {
    /** Send the current configuration/information */
    SendConfiguration,
    /** Rewind replay to the indicated position */
    SpoolReplay,
    /** Start replaying at the given time */
    StartReplay,
    /** Complete the recording at the given time */
    CompleteRecording,
    /** Provide a name for the upcoming recording */
    NameRecording,
    /** Set the tick time for the start of the next
        recording */
    StartRecording,
    /** Check whether recording is complete, flush to file
        and if so report on the new stretch */
    FlushAndCollect,
    /** periodic flush to file */
    FlushToDisk,
    /** load buffers from file */
    FillReplayBuffers
  };

  /** The command (currently only one option) */
  Command command;

  /** Run generation to select for the data */
  uint32_t run_cycle;

  /** Tick value to reset to */
  TimeTickType tick;

  /** String value */
  std::string sdata;

  /** String value 2 */
  std::string sdata2;

public:
  /** a "magic" number, hashed out of the class definition,
      that will be used to check consistency of the sent objects
      across the dueca nodes. */
  static const uint32_t magic_check_number;

  /** default constructor. */
  ReplayCommand();

  /** Constructor with arguments */
  ReplayCommand(
        const Command& command,
        const uint32_t& run_cycle,
        const TimeTickType& tick,
        const std::string& sdata,
        const std::string& sdata2);

  /** copy constructor. */
  ReplayCommand(const ReplayCommand& o);

#if !defined(__DCO_NOPACK)
  /** constructor to restore an ReplayCommand from amorphous storage. */
  ReplayCommand(::dueca::AmorphReStore& r);
#endif

  /** destructor. */
  ~ReplayCommand();

#if !defined(__DCO_STANDALONE)
  /** new operator "new", which places objects not on a
      heap, but in one of the memory arenas. This to speed up
      memory management. */
  static void* operator new(size_t size);

  /** new operator "delete", to go with the new version
      of operator new. */
  static void operator delete(void* p);

  /** placement "new", needed for stl. */
  inline static void* operator new(size_t size, ReplayCommand*& o)
  { return reinterpret_cast<void*>(o); }
#endif

#if !defined(__DCO_NOPACK)
  /** packs the ReplayCommand into amorphous storage. */
  void packData(::dueca::AmorphStore& s) const;

  /** packs the ReplayCommand into amorphous storage.
      only differences with a previous object are packed. */
  void packDataDiff(::dueca::AmorphStore& s, const ReplayCommand& ref) const;

  /** unpacks the ReplayCommand from an amorphous storage. */
  void unPackData(::dueca::AmorphReStore& s);

  /** unpacks the differences for ReplayCommand
      from an amorphous storage. */
  void unPackDataDiff(::dueca::AmorphReStore& s);
#endif

  /** Test for equality. */
  bool operator == (const ReplayCommand& o) const;

  /** Test for inequality. */
  inline bool operator != (const ReplayCommand& o) const
  { return !(*this == o); }

  /** Assignment operator. */
  ReplayCommand& operator=(const ReplayCommand& o);

  /** prints the ReplayCommand to a stream. */
  std::ostream & print(std::ostream& s) const;
};

} 

#if !defined(__DCO_NOPACK)
/** pack the object into amorphous storage. */
inline void packData(::dueca::AmorphStore& s,
                     const dueca::ReplayCommand& o)
{ o.packData(s); }

/** pack the differences between this object and another
    into amorphous storage. */
inline void packDataDiff(dueca::AmorphStore& s,
                         const dueca::ReplayCommand& o,
                         const dueca::ReplayCommand& ref)
{ o.packDataDiff(s, ref); }

/** unpack the object from amorphous storage. */
inline void unPackData(::dueca::AmorphReStore& s,
                       dueca::ReplayCommand& o)
{ o.unPackData(s); }

/** unpack the differences to this object from storage. */
inline void unPackDataDiff(dueca::AmorphReStore& s,
                           dueca::ReplayCommand& o)
{ o.unPackDataDiff(s); }
#endif

namespace dueca {
  const char* const getString(const dueca::ReplayCommand::Command &o);
  void readFromString(dueca::ReplayCommand::Command &o, const std::string& s);
  void getFirst(dueca::ReplayCommand::Command &o);
  bool getNext(dueca::ReplayCommand::Command &o);

  /** This function retrieves the classname of an ReplayCommand::Command */
  template <>
  const char* getclassname<dueca::ReplayCommand::Command>();
}; // end namespace dueca

#if !defined(__DCO_NOPACK)
void packData(::dueca::AmorphStore& s,
              const dueca::ReplayCommand::Command &o);
void unPackData(::dueca::AmorphReStore& s,
                dueca::ReplayCommand::Command &o);
#endif

PRINT_NS_START;
/** Print an object of type ReplayCommand::Command */
inline std::ostream&
operator << (std::ostream& s, const dueca::ReplayCommand::Command& o)
{ return s << dueca::getString(o); }
/** Read an object of type ReplayCommand::Command */
inline std::istream&
/** Read an enum of type ReplayCommand::Command */
operator >> (std::istream& s, dueca::ReplayCommand::Command& o)
{ std::string tmp; s >> tmp; dueca::readFromString(o, tmp); return s; }
PRINT_NS_END;

namespace std {
/** print to a stream. */
inline std::ostream &
operator << (std::ostream& s, const dueca::ReplayCommand& o)
{ return o.print(s); }
};

// specialization of classname
template<>
const char* dueca::getclassname<dueca::ReplayCommand>();
template <>
inline const char* dueca::getclassname<const dueca::ReplayCommand>()
{ return dueca::getclassname<dueca::ReplayCommand>(); }

#if !defined(__DCO_STANDALONE)
namespace dueca {
/** Template specialization, defines a trait that is needed if
    ReplayCommand is ever used inside other dco objects. */
template <>
struct dco_nested<dueca::ReplayCommand> : public dco_isnested { };
template <>
struct dco_nested<dueca::ReplayCommand::Command> : public dco_isenum { };
};

#endif


