/* ------------------------------------------------------------------ */
/*      item            : ReplicatorPeerAcknowledge.hxx
        generated by    : Autobuild (abuild)
        date            : Thu 09 Nov 2023
        category        : header file
        description     : DUECA Communication Object (DCO)
                          automatically generated by dueca-codegen
        codegen version : 111
        language        : C++
        original item   : ReplicatorPeerAcknowledge.dco
        made by         : Rene' van Paassen
        date            : 170205
        description     : Peer information replicator nodes
        documentation   : DUECA_API
        copyright       : (c) 2017-2022 TUDelft-AE-C&S
                        : (c) 2022 René van Paassen
        license         : EUPL-1.2
*/

#pragma once
#if !defined(__DCO_NOPACK)
namespace dueca {
class AmorphStore;
class AmorphReStore;
struct DataWriterArraySize;
};
#endif

#if !defined(__DCO_STANDALONE)
namespace dueca {
struct CommObjectDataTable;
};

#include <gencodegen.h>
#if GENCODEGEN != 111
#error "Generated ReplicatorPeerAcknowledge.hxx too old, please clean with 'make mrproper'"
#endif
#include <dueca/CommObjectTraits.hxx>
#endif
#include <iostream>

#include <stringoptions.h>
#include <inttypes.h>

namespace dueca {
 
/** Start information for a peer joining a replicator network.
   
    With an object of this type, the application at the master end can
    accept or reject peer connections. Data for the peer (limited to 1K)
    can be added.
   
    At the peer end, this informs of the ID issued by the master, and of
    optionally peer data from the application
    */
struct ReplicatorPeerAcknowledge
{
  /** typedef for internal reference */
  typedef ReplicatorPeerAcknowledge __ThisDCOType__;

public:
  /** The name of this class. */
  static const char* const classname;


  /** Data for the peer, if the master replicator has a channel
      connection to an application module that vets peer connections,
      the peer data returned by that module is passed to the newly
      connected peer */
  vstring peerdata;

  /** issued peer id -- for matching response */
  uint16_t peer_id;

  /** rejection flag */
  bool reject;

public:
  /** a "magic" number, hashed out of the class definition,
      that will be used to check consistency of the sent objects
      across the dueca nodes. */
  static const uint32_t magic_check_number;

  /** default constructor. */
  ReplicatorPeerAcknowledge();

  /** Constructor with arguments */
  ReplicatorPeerAcknowledge(
        const vstring& peerdata,
        const uint16_t& peer_id,
        const bool& reject);

  /** copy constructor. */
  ReplicatorPeerAcknowledge(const ReplicatorPeerAcknowledge& o);

#if !defined(__DCO_NOPACK)
  /** constructor to restore an ReplicatorPeerAcknowledge from amorphous storage. */
  ReplicatorPeerAcknowledge(::dueca::AmorphReStore& r);
#endif

  /** destructor. */
  ~ReplicatorPeerAcknowledge();

#if !defined(__DCO_STANDALONE)
  /** new operator "new", which places objects not on a
      heap, but in one of the memory arenas. This to speed up
      memory management. */
  static void* operator new(size_t size);

  /** new operator "delete", to go with the new version
      of operator new. */
  static void operator delete(void* p);

  /** placement "new", needed for stl. */
  inline static void* operator new(size_t size, ReplicatorPeerAcknowledge*& o)
  { return reinterpret_cast<void*>(o); }
#endif

#if !defined(__DCO_NOPACK)
  /** packs the ReplicatorPeerAcknowledge into amorphous storage. */
  void packData(::dueca::AmorphStore& s) const;

  /** packs the ReplicatorPeerAcknowledge into amorphous storage.
      only differences with a previous object are packed. */
  void packDataDiff(::dueca::AmorphStore& s, const ReplicatorPeerAcknowledge& ref) const;

  /** unpacks the ReplicatorPeerAcknowledge from an amorphous storage. */
  void unPackData(::dueca::AmorphReStore& s);

  /** unpacks the differences for ReplicatorPeerAcknowledge
      from an amorphous storage. */
  void unPackDataDiff(::dueca::AmorphReStore& s);
#endif

  /** Test for equality. */
  bool operator == (const ReplicatorPeerAcknowledge& o) const;

  /** Test for inequality. */
  inline bool operator != (const ReplicatorPeerAcknowledge& o) const
  { return !(*this == o); }

  /** Assignment operator. */
  ReplicatorPeerAcknowledge& operator=(const ReplicatorPeerAcknowledge& o);

  /** prints the ReplicatorPeerAcknowledge to a stream. */
  std::ostream & print(std::ostream& s) const;
};

} 

#if !defined(__DCO_NOPACK)
/** pack the object into amorphous storage. */
inline void packData(::dueca::AmorphStore& s,
                     const dueca::ReplicatorPeerAcknowledge& o)
{ o.packData(s); }

/** pack the differences between this object and another
    into amorphous storage. */
inline void packDataDiff(dueca::AmorphStore& s,
                         const dueca::ReplicatorPeerAcknowledge& o,
                         const dueca::ReplicatorPeerAcknowledge& ref)
{ o.packDataDiff(s, ref); }

/** unpack the object from amorphous storage. */
inline void unPackData(::dueca::AmorphReStore& s,
                       dueca::ReplicatorPeerAcknowledge& o)
{ o.unPackData(s); }

/** unpack the differences to this object from storage. */
inline void unPackDataDiff(dueca::AmorphReStore& s,
                           dueca::ReplicatorPeerAcknowledge& o)
{ o.unPackDataDiff(s); }
#endif

namespace std {
/** print to a stream. */
inline std::ostream &
operator << (std::ostream& s, const dueca::ReplicatorPeerAcknowledge& o)
{ return o.print(s); }
};

// specialization of classname
template<>
const char* dueca::getclassname<dueca::ReplicatorPeerAcknowledge>();
template <>
inline const char* dueca::getclassname<const dueca::ReplicatorPeerAcknowledge>()
{ return dueca::getclassname<dueca::ReplicatorPeerAcknowledge>(); }

#if !defined(__DCO_STANDALONE)
namespace dueca {
/** Template specialization, defines a trait that is needed if
    ReplicatorPeerAcknowledge is ever used inside other dco objects. */
template <>
struct dco_nested<dueca::ReplicatorPeerAcknowledge> : public dco_isnested { };
};

#endif


