/* ------------------------------------------------------------------ */
/*      item            : UDPPeerConfig.hxx
        generated by    : Autobuild (abuild)
        date            : Thu 09 Nov 2023
        category        : header file
        description     : DUECA Communication Object (DCO)
                          automatically generated by dueca-codegen
        codegen version : 111
        language        : C++
        original item   : UDPPeerConfig.dco
        made by         : Rene' van Paassen
        date            : 170205
        description     : Configuration/Coordination replicator nodes
        copyright       : (c) 2017-2022 TUDelft-AE-C&S
                        : (c) 2022 René van Paassen
        license         : EUPL-1.2
*/

#pragma once
#if !defined(__DCO_NOPACK)
namespace dueca {
class AmorphStore;
class AmorphReStore;
struct DataWriterArraySize;
};
#endif

#if !defined(__DCO_STANDALONE)
namespace dueca {
struct CommObjectDataTable;
};

#include <gencodegen.h>
#if GENCODEGEN != 111
#error "Generated UDPPeerConfig.hxx too old, please clean with 'make mrproper'"
#endif
#include <dueca/CommObjectTraits.hxx>
#endif
#include <iostream>

#include <inttypes.h>
typedef uint16_t peer_id_t;
#include <inttypes.h>

namespace dueca {
 
/** Configuration message for replicator coordination.
    */
struct UDPPeerConfig
{
  /** typedef for internal reference */
  typedef UDPPeerConfig __ThisDCOType__;

public:
  /** The name of this class. */
  static const char* const classname;


/** defines type of message
 */
  enum MessageType {
    /** Hook up to preceding, data: peer_id of preceding in message chain
        and cycle at which the hookup should start, 0 if immediate */
    HookUp,
    /** Peer leave message, data: peer_id and cycle at which peer leaves,
        0 if ASAP */
    DeletePeer,
    /** Configure new peer, data: peer_id */
    ConfigurePeer,
    /** Initial configuration burst complete, no data */
    InitialConfComplete,
    /** Client payload follows (from derived class), no data */
    ClientPayload,
    /** Undefined, no data */
    Undefined
  };

  /** type of config message */
  MessageType mtype;

  /** peer id */
  peer_id_t peer_id;

  /** message cycle selected for change */
  uint32_t target_cycle;

public:
  /** a "magic" number, hashed out of the class definition,
      that will be used to check consistency of the sent objects
      across the dueca nodes. */
  static const uint32_t magic_check_number;

  /** default constructor. */
  UDPPeerConfig(
        const MessageType& mtype = Undefined,
        const peer_id_t& peer_id = 0U,
        const uint32_t& target_cycle = 0U);

  /** copy constructor. */
  UDPPeerConfig(const UDPPeerConfig& o);

#if !defined(__DCO_NOPACK)
  /** constructor to restore an UDPPeerConfig from amorphous storage. */
  UDPPeerConfig(::dueca::AmorphReStore& r);
#endif

  /** destructor. */
  ~UDPPeerConfig();

#if !defined(__DCO_STANDALONE)
  /** new operator "new", which places objects not on a
      heap, but in one of the memory arenas. This to speed up
      memory management. */
  static void* operator new(size_t size);

  /** new operator "delete", to go with the new version
      of operator new. */
  static void operator delete(void* p);

  /** placement "new", needed for stl. */
  inline static void* operator new(size_t size, UDPPeerConfig*& o)
  { return reinterpret_cast<void*>(o); }
#endif

#if !defined(__DCO_NOPACK)
  /** packs the UDPPeerConfig into amorphous storage. */
  void packData(::dueca::AmorphStore& s) const;

  /** packs the UDPPeerConfig into amorphous storage.
      only differences with a previous object are packed. */
  void packDataDiff(::dueca::AmorphStore& s, const UDPPeerConfig& ref) const;

  /** unpacks the UDPPeerConfig from an amorphous storage. */
  void unPackData(::dueca::AmorphReStore& s);

  /** unpacks the differences for UDPPeerConfig
      from an amorphous storage. */
  void unPackDataDiff(::dueca::AmorphReStore& s);
#endif

  /** Test for equality. */
  bool operator == (const UDPPeerConfig& o) const;

  /** Test for inequality. */
  inline bool operator != (const UDPPeerConfig& o) const
  { return !(*this == o); }

  /** Assignment operator. */
  UDPPeerConfig& operator=(const UDPPeerConfig& o);

  /** prints the UDPPeerConfig to a stream. */
  std::ostream & print(std::ostream& s) const;

// Additional custom code
#include "UDPPeerConfigExtra.hxx"
};

} 

#if !defined(__DCO_NOPACK)
/** pack the object into amorphous storage. */
inline void packData(::dueca::AmorphStore& s,
                     const dueca::UDPPeerConfig& o)
{ o.packData(s); }

/** pack the differences between this object and another
    into amorphous storage. */
inline void packDataDiff(dueca::AmorphStore& s,
                         const dueca::UDPPeerConfig& o,
                         const dueca::UDPPeerConfig& ref)
{ o.packDataDiff(s, ref); }

/** unpack the object from amorphous storage. */
inline void unPackData(::dueca::AmorphReStore& s,
                       dueca::UDPPeerConfig& o)
{ o.unPackData(s); }

/** unpack the differences to this object from storage. */
inline void unPackDataDiff(dueca::AmorphReStore& s,
                           dueca::UDPPeerConfig& o)
{ o.unPackDataDiff(s); }
#endif

namespace dueca {
  const char* const getString(const dueca::UDPPeerConfig::MessageType &o);
  void readFromString(dueca::UDPPeerConfig::MessageType &o, const std::string& s);
  void getFirst(dueca::UDPPeerConfig::MessageType &o);
  bool getNext(dueca::UDPPeerConfig::MessageType &o);

  /** This function retrieves the classname of an UDPPeerConfig::MessageType */
  template <>
  const char* getclassname<dueca::UDPPeerConfig::MessageType>();
}; // end namespace dueca

#if !defined(__DCO_NOPACK)
void packData(::dueca::AmorphStore& s,
              const dueca::UDPPeerConfig::MessageType &o);
void unPackData(::dueca::AmorphReStore& s,
                dueca::UDPPeerConfig::MessageType &o);
#endif

PRINT_NS_START;
/** Print an object of type UDPPeerConfig::MessageType */
inline std::ostream&
operator << (std::ostream& s, const dueca::UDPPeerConfig::MessageType& o)
{ return s << dueca::getString(o); }
/** Read an object of type UDPPeerConfig::MessageType */
inline std::istream&
/** Read an enum of type UDPPeerConfig::MessageType */
operator >> (std::istream& s, dueca::UDPPeerConfig::MessageType& o)
{ std::string tmp; s >> tmp; dueca::readFromString(o, tmp); return s; }
PRINT_NS_END;

namespace std {
/** print to a stream. */
inline std::ostream &
operator << (std::ostream& s, const dueca::UDPPeerConfig& o)
{ return o.print(s); }
};

// specialization of classname
template<>
const char* dueca::getclassname<dueca::UDPPeerConfig>();
template <>
inline const char* dueca::getclassname<const dueca::UDPPeerConfig>()
{ return dueca::getclassname<dueca::UDPPeerConfig>(); }

#if !defined(__DCO_STANDALONE)
namespace dueca {
/** Template specialization, defines a trait that is needed if
    UDPPeerConfig is ever used inside other dco objects. */
template <>
struct dco_nested<dueca::UDPPeerConfig> : public dco_isnested { };
template <>
struct dco_nested<dueca::UDPPeerConfig::MessageType> : public dco_isenum { };
};

#endif


